/* mess_avc: UNIX command-/event-driven audio-volume-controller.
 * Module: audio-device routines.
 * @(#)audctl.c	1.2	20jan2013 MJS
 *
 * Copyright (c) 2013 by Mike Spooner
 *----------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------
 *
 * Core API:    ANSI X3.159-1989 C Language
 * Other APIs:  MJS Portable C Library
 *		SunOS 5.x audio(7i) interface
 ***********************************************************************
 */
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include "mjsu.h"
#include "audctl.h"

#define VOL_MAX 100	/* sic */
#define VOL_MIN 0	/* sic */

#if defined(__sun) && defined(__SVR4)
#define __EXTENSIONS__
#include <sys/audioio.h>
#define AUDIODEV	"/dev/audio"	/* Solaris default device */
#define CTL_SFX	"ctl"	/* device-name suffix for audio-*control*-devices */

/* NOTE: the Sun audioio API specifies a *linear* "gain", rather than 
 * a decibel scale.
 */
#define AUDIO_SPAN (AUDIO_MAX_GAIN - AUDIO_MIN_GAIN)

INT audioctl_open(void)
	{
	INT fd;
	CHAR *pathname = NULL, *p;
	struct audio_info inf;

	/* construct pathname of the audio-control device: */
	if ((p = getenv("AUDIODEV")))
		;
	else
		p = AUDIODEV;

	pathname = mem_buy(NULL, strlen(p) + strlen(CTL_SFX) + 1, YES);
	sprintf(pathname, "%s%s", p, CTL_SFX);

	if ((fd = open(pathname, O_RDWR|O_NOCTTY, 0)) < 0)
		{
		mem_free(pathname);
		error("cannot open audio-control device: %s", pathname);
		}
	if (ioctl(fd, AUDIO_GETINFO, &inf) < 0)
		{
		mem_free(pathname);
		error("cannot get audio settings from %s", pathname);
		}
	return (fd);
	}

INT audioctl_getvol(INT fd)
	{
	INT vol;
	double v;
	struct audio_info inf;

	if (ioctl(fd, AUDIO_GETINFO, &inf) < 0)
		return (0);
	v = inf.play.gain;
	v -= AUDIO_MIN_GAIN;
	v *= VOL_MAX;
	v /= AUDIO_SPAN;

	vol = min(v, VOL_MAX);
	vol = max(vol, VOL_MIN);
	return (vol);
	}


BOOL audioctl_setvol(INT fd, INT vol)
	{
	struct audio_info inf;
	double gain;

	vol = min(vol, VOL_MAX);
	vol = max(vol, VOL_MIN);

	gain = vol;
	gain *= AUDIO_SPAN;
	gain /= VOL_MAX;
	gain += AUDIO_MIN_GAIN;
	gain = max(gain, AUDIO_MIN_GAIN);	/* paranoia */
	gain = min(gain, AUDIO_MAX_GAIN);	/* paranoia */

	inf.play.gain = gain;
	if (ioctl(fd, AUDIO_SETINFO, &inf) < 0)
		return (NO);
	return (YES);
	}

INT audioctl_close(INT fd)
	{
	if (fd >= 0)
		close(fd);
	return (-1);
	}
#else
#error "unrecognised platform"
#endif
